// src/filesystem/project-root.ts
import { dirname } from "node:path";
import { findUp } from "find-up";
async function projectRoot(fallbackToCwd = false, cwd) {
  const packageJsonPath = await findUp("package.json", { cwd });
  if (!packageJsonPath) {
    if (fallbackToCwd) {
      return process.cwd();
    }
    throw new Error("Unable to find project root (no package.json found)");
  }
  return dirname(packageJsonPath);
}

// src/filesystem/exists.ts
import { stat } from "node:fs/promises";
async function exists(path) {
  try {
    await stat(path);
    return true;
  } catch {
    return false;
  }
}

// src/filesystem/mono-repo.ts
import { readFile } from "node:fs/promises";
import { dirname as dirname2, join } from "node:path";

// src/json.ts
function tryParseJson(value, defaultValue = null) {
  try {
    const parsed = JSON.parse(value);
    if (parsed === void 0 || parsed === null) {
      return defaultValue;
    }
    return parsed;
  } catch (err) {
    return defaultValue;
  }
}

// src/filesystem/mono-repo.ts
import { findUp as findUp2 } from "find-up";
import { glob } from "glob";
async function findMonoRepoRoot(startDir) {
  const lockFilePath = await findUp2(["package-lock.json", "yarn.lock", "pnpm-lock.yaml", "bun.lockb", "bun.lock"], {
    cwd: startDir
  });
  if (lockFilePath) {
    const packageJsonPath = join(dirname2(lockFilePath), "package.json");
    const hasWorkSpaces = await packageJsonHasWorkspaces(packageJsonPath);
    return hasWorkSpaces ? dirname2(lockFilePath) : null;
  }
  let currentDir = startDir;
  while (currentDir !== "/") {
    const packageJsonPath = join(currentDir, "package.json");
    if (await packageJsonHasWorkspaces(packageJsonPath)) {
      return currentDir;
    }
    const parentDir = dirname2(currentDir);
    if (parentDir === currentDir) {
      break;
    }
    currentDir = parentDir;
  }
  return null;
}
async function findMonoRepoPackages(projectDir) {
  try {
    const monoRepoRoot = await findMonoRepoRoot(projectDir);
    if (!monoRepoRoot) {
      return [projectDir];
    }
    const packageJsonPath = join(monoRepoRoot, "package.json");
    const packageJson = tryParseJson(await readFile(packageJsonPath, "utf-8"));
    const workspaces = packageJson?.workspaces ?? [];
    const packagePaths = await Promise.all(
      workspaces.map(async (workspace) => {
        const matches = await glob(join(monoRepoRoot, workspace, "package.json"));
        return matches.map((match) => join(match, ".."));
      })
    );
    const allPaths = packagePaths.flat();
    return allPaths.length === 0 ? [projectDir] : [monoRepoRoot, ...allPaths];
  } catch (error) {
    return [projectDir];
  }
}
async function packageJsonHasWorkspaces(packageJsonPath) {
  if (await exists(packageJsonPath)) {
    const packageJson = tryParseJson(await readFile(packageJsonPath, "utf-8"));
    if (packageJson?.workspaces && Array.isArray(packageJson?.workspaces) && packageJson?.workspaces.length > 0) {
      return true;
    }
  }
  return false;
}
export {
  exists,
  findMonoRepoPackages,
  findMonoRepoRoot,
  projectRoot
};
//# sourceMappingURL=filesystem.mjs.map